#ifndef HPCE_TXNREADER_H
#define HPCE_TXNREADER_H

#include <sstream>
#include <fstream>

#include "..\..\inc\EGenStandardTypes.h"
#include "..\..\inc\DriverParamSettings.h"
#include "..\..\inc\EGenVersion.h"
#include "..\..\inc\BaseLogger.h"
#include "..\..\inc\locking.h"

namespace HPZREF
{

class CHPCETxnReader 
{
private:
    char        m_Filename[512];    ifstream    m_File;
    CMutex      m_FileLock;

public:
    CHPCETxnReader(const char *szFilename)
    {
        // Copy Log Filename
        strncpy(m_Filename, szFilename, sizeof(m_Filename));

        // Open Log File
        m_File.open(m_Filename);
        if (!m_File)
        {
            throw CSystemErr(CSystemErr::eCreateFile, "HPCETxnFile::HPCETxnFile");
        }
    };

	~CHPCETxnReader()
	{
		if (m_File) m_File.close();
	}

	bool WriteTxn(const char *szPrefix, const char *szTimestamp, const char *txn, const long txnSize)
    {
        m_FileLock.lock();
		m_File << szPrefix ;
		m_File.write(txn, txnSize);
        m_File << endl;
        m_File.flush();
        if (!m_File)
        {
            throw CSystemErr(CSystemErr::eWriteFile, "HPCETxnFile::WriteTxn");
        }
        m_FileLock.unlock();
        return true;
    }

	bool WriteString(string txt)
    {
        m_FileLock.lock();
		m_File << txt;
        m_File.flush();
        if (!m_File)
        {
            throw CSystemErr(CSystemErr::eWriteFile, "HPCETxnFile::WriteString");
        }
        m_FileLock.unlock();
        return true;
    }

};

}   // namespace HPZREF

#endif //HPCE_TXNREADER_H
