#include "StdAfx.h"

CMSMQReceiver::CMSMQReceiver(void) 
	: m_pLabelMSMQ(NULL)
	, m_hQueue(NULL)
	, m_pMsgProps(NULL)
	, m_pMsg(NULL)
	, m_pLabel(NULL)
	, m_MaxLabelLength(0)
	, m_MaxMsgSize(0)
{
}

CMSMQReceiver::~CMSMQReceiver(void)
{
	wprintf(L"MQReceiver destructor. Thread Id: %x. Class: %x\n", GetCurrentThreadId(), this);

	if (m_pLabelMSMQ) delete[] m_pLabelMSMQ;

	if (m_pMsgProps)
	{
		if (m_pMsgProps->aPropID)  delete [] m_pMsgProps->aPropID;
		if (m_pMsgProps->aPropVar) delete [] m_pMsgProps->aPropVar;
		if (m_pMsgProps->aStatus)  delete [] m_pMsgProps->aStatus;
		delete[] m_pMsgProps;
	}

}


HRESULT CMSMQReceiver::Initialize(char * pComputerName, char * pQueueName, 
		void * pMsg, int MaxMsgSize, char * pLabel, int MaxLabelLength)
{
	int cThread = 0, cReceive = 0;
	HANDLE hPort = NULL;
	QUEUEHANDLE hQueue = NULL;
	HRESULT hr = MQ_OK;

	CUTF queueName(pQueueName);
	CUTF computerName(pComputerName);
	
	LPWSTR wszFormatName = CMSMQUtil::GetQueueFormatName(queueName.get16(), computerName.get16());		

	//	Open the queue for the process.
	//if (m_hQueueRead == NULL)
	if (m_hQueue == NULL)
	{
		hr = MQOpenQueue(
							wszFormatName,
							MQ_RECEIVE_ACCESS,
							MQ_DENY_NONE,
							//&m_hQueueRead
							&m_hQueue
							);
		delete [] wszFormatName;
		if (FAILED(hr))
		{
			wprintf(L"MQOpenQueue failed. Error: 0x%X. Thread Id: %x\n", hr, GetCurrentThreadId());
			return hr;
		}
	}

	//	Set up caller-facing message-related buffers and info
	m_pLabel = pLabel;
	m_MaxLabelLength = MaxLabelLength;
	
	m_pMsg = pMsg;
	m_MaxMsgSize = MaxMsgSize;
	
	//	Create internal message-related buffers
	m_pLabelMSMQ = new WCHAR[MQ_MAX_MSG_LABEL_LEN];

	//	Set up message properties for receiving.  
	m_pMsgProps = new MQMSGPROPS;
	if (m_pMsgProps == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;

	m_pMsgProps->aPropID = new MSGPROPID[R_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aPropID == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;
		
	m_pMsgProps->aPropVar = new MQPROPVARIANT[R_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aPropVar == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;
		
	m_pMsgProps->aStatus = new HRESULT[R_NUMBEROFPROPERTIES];
	if (m_pMsgProps->aStatus == NULL) return MQ_ERROR_INSUFFICIENT_RESOURCES;
		
	//	Label properties
	DWORD cPropId = R_ARRAY_PROP_LABEL_LEN;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_LABEL_LEN;
	m_pMsgProps->aPropVar[cPropId].vt = VT_UI4;
	m_pMsgProps->aPropVar[cPropId].ulVal = MQ_MAX_MSG_LABEL_LEN;
    
	cPropId = R_ARRAY_PROP_LABEL;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_LABEL;
	m_pMsgProps->aPropVar[cPropId].vt = VT_LPWSTR;
	m_pMsgProps->aPropVar[cPropId].pwszVal = m_pLabelMSMQ;
	
	//	Message body properties
	//	Size - will be returned on Receive
	cPropId = R_ARRAY_PROP_BODY_SIZE;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_BODY_SIZE;
	m_pMsgProps->aPropVar[cPropId].vt = VT_UI4;  //	TODO: VT_NULL?

	//	Buffer - uses passed-in buffer
	cPropId = R_ARRAY_PROP_BODY;
	m_pMsgProps->aPropID[cPropId] = PROPID_M_BODY;
	m_pMsgProps->aPropVar[cPropId].vt = VT_VECTOR | VT_UI1;
	m_pMsgProps->aPropVar[cPropId].caub.pElems = (UCHAR *) m_pMsg;
	m_pMsgProps->aPropVar[cPropId].caub.cElems = m_MaxMsgSize;

	m_pMsgProps->cProp = R_NUMBEROFPROPERTIES;

	return hr;

}

HRESULT CMSMQReceiver::Receive(DWORD timeout)
{
	HRESULT hr = MQ_OK;

	hr =  MQReceiveMessage(
    //            m_hQueueRead,           // Queue handle
	            m_hQueue,           // Queue handle
                timeout,				// Time to wait for a message
                MQ_ACTION_RECEIVE,
                m_pMsgProps,      // Pointer to an MQMSGPROPS structure
                NULL,		           // Synchronous, no OVERLAPPED structure 
                NULL,					// No callback function
                NULL,					// No cursor
                MQ_NO_TRANSACTION		// Not in a transaction
                );

	if (FAILED(hr))
	{
		if (hr == MQ_ERROR_IO_TIMEOUT)
			wprintf(L"MQReceiveMessage Timeout. Thread Id: %x\n", GetCurrentThreadId());
		else
			wprintf(L"MQReceiveMessage failed. Error: 0x%X. Thread Id: %x\n", hr, GetCurrentThreadId());

		return hr;
	}

	//	Convert to ANSI, no longer than the passed-in buffer
	CUTF label(m_pLabelMSMQ);

	hr = StringCbCopyA(m_pLabel, m_MaxLabelLength, label.get8());
	if (hr == STRSAFE_E_INSUFFICIENT_BUFFER)
	{
		//	TODO: Label longer than caller expected
	}
	else if (FAILED(hr))	
	{
		return (hr);
	}

	//	Body - transaction data - already placed in m_pMsg
	unsigned long bodySize = m_pMsgProps->aPropVar[R_ARRAY_PROP_BODY_SIZE].ulVal;

    wprintf(L"Message received. Thread ID: %x. Body size: %lu\n",
            GetCurrentThreadId(),
            bodySize
            );
	
	return hr;
	
}

HRESULT CMSMQReceiver::Shutdown(void)
{
	HRESULT hr = MQ_OK;

	if (m_hQueue)
	{
		hr = MQCloseQueue(m_hQueue);
		if (FAILED(hr))
			wprintf(L"MQCloseQueue failed. Error: 0x%X. Thread Id: %x\n", hr, GetCurrentThreadId());

		m_hQueue = NULL;
	}

	return hr;
}


