#ifndef HP_TXNLOGFORMATTER_H
#define HP_TXNLOGFORMATTER_H

#include <sstream>
#include <fstream>
#include <string>
#include <iomanip>

#include "../../TPC-E/inc/TxnHarnessStructs.h"
//#include "../../TPC-E/inc/EGenStandardTypes.h"
//#include "../../TPC-E/inc/EGenVersion.h"
//#include "../../TPC-E/inc/locking.h"
//#include "../../TPC-E/inc/DateTime.h"
#define datetime_fmtcode 12

namespace HPZREF
{

//	string emptyString("");

class CHPTxnLogFormatter
{
private:

	string FormatTS(TIMESTAMP_STRUCT structTS, const INT32 dtFormat)
	{
		string dt;
		if (structTS.year == 0 && structTS.hour == 0)
		{
			dt = "";
		}
		else
		{
			CDateTime * pDT1 = new CDateTime(&structTS);
			dt = pDT1->ToStr(dtFormat);
		}

		return dt;
	}

public:
    CHPTxnLogFormatter()
    {
    };

	~CHPTxnLogFormatter()
	{
	};


	string CHPTxnLogFormatter::GetLogOutput(TBrokerVolumeTxnInput * pParms)
	{
		ostringstream logmsg;
		int idx = 0;

		// Construct Log String
		logmsg << "BV Broker Volume Txn:" << endl;
		for (idx = 0; idx < max_broker_list_len; idx++)
		{
			if (strlen(pParms->broker_list[idx]) > 0)
			{
				logmsg << "   Broker Name " << idx << ":" << "\t" << pParms->broker_list[idx] << endl;
			}
		}
		logmsg << "   Sector Name: " << "\t" << pParms->sector_name << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TCustomerPositionTxnInput * pParms)
	{

		ostringstream logmsg;
		// Construct Log String
		logmsg << "CP Customer Position Txn:" << endl;
		logmsg << "   Acct ID Idx:    " << "\t" << pParms->acct_id_idx << endl;
		logmsg << "   Cust ID:        " << "\t" << pParms->cust_id << endl;
		logmsg << "   Get History:    " << "\t" << pParms->get_history << endl;
		logmsg << "   Tax ID:         " << "\t" << pParms->tax_id << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TMarketWatchTxnInput * pParms)
	{

		ostringstream logmsg;
		// Construct Log String
		logmsg << "MW Market Watch Txn:" << endl;
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << "   C ID:           " << "\t" << pParms->c_id << endl;
		logmsg << "   End Co ID:      " << "\t" << pParms->ending_co_id << endl;
		logmsg << "   Start Co ID:    " << "\t" << pParms->starting_co_id << endl;
		logmsg << "   Start Day:      " << "\t" << FormatTS(pParms->start_day, dateonly_fmtcode) << endl;
		logmsg << "   Industry Name:  " << "\t" << pParms->industry_name << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TSecurityDetailTxnInput * pParms)
	{

		ostringstream logmsg;
		// Construct Log String
		logmsg << "SD Security Detail Txn:" << endl;
		logmsg << "   Max Rows:       " << "\t" << pParms->max_rows_to_return << endl;
		logmsg << "   Access LOB Flag:" << "\t" << pParms->access_lob_flag << endl;
		logmsg << "   Start Day:      " << "\t" << FormatTS(pParms->start_day, dateonly_fmtcode) << endl;
		logmsg << "   Symbol:         " << "\t" << pParms->symbol << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TTradeLookupTxnInput * pParms)
	{

		int idx = 0;
 
		ostringstream logmsg;
		// Construct Log String
		logmsg << "TL Trade Lookup Txn:" << endl;
		for (idx = 0; idx < TradeLookupFrame1MaxRows; idx++)
		{
			if (pParms->trade_id[idx] != 0)
			{
				logmsg << "   Trade ID " << idx << ":    " << "\t" << pParms->trade_id[idx] << endl;
			}
		}
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << "   Max Acct ID:    " << "\t" << pParms->max_acct_id << endl;
		logmsg << "   Frame To Exec:  " << "\t" << pParms->frame_to_execute << endl;
		logmsg << "   Max Trades:     " << "\t" << pParms->max_trades << endl;
		logmsg << "   End Trade Dt:   " << "\t" << FormatTS(pParms->end_trade_dts, datetime_fmtcode) << endl;
		logmsg << "   Start Trade Dt: " << "\t" << FormatTS(pParms->start_trade_dts, datetime_fmtcode) << endl;
		logmsg << "   Symbol:         " << "\t" << pParms->symbol << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TTradeOrderTxnInput * pParms)
	{

		ostringstream logmsg;
		// Construct Log String
		logmsg << "TO Trade Order Txn:" << endl;
		logmsg << "   Requested Price:" << "\t" << setprecision(2) << pParms->requested_price << endl;
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << "   Is LIFO:        " << "\t" << pParms->is_lifo << endl;
		logmsg << "   Roll It Back:   " << "\t" << pParms->roll_it_back << endl;
		logmsg << "   Trade Quantity: " << "\t" << pParms->trade_qty << endl;
		logmsg << "   Type is Margin: " << "\t" << pParms->type_is_margin << endl;
		logmsg << "   Co Name:        " << "\t" << pParms->co_name << endl;
		logmsg << "   Exec First Name:" << "\t" << pParms->exec_f_name << endl;
		logmsg << "   Exec Last Name: " << "\t" << pParms->exec_l_name << endl;
		logmsg << "   Exec Tax ID:    " << "\t" << pParms->exec_tax_id << endl;
		logmsg << "   Issue:          " << "\t" << pParms->issue << endl;
		logmsg << "   ST Pending ID:  " << "\t" << pParms->st_pending_id << endl;
		logmsg << "   ST Submitted ID:" << "\t" << pParms->st_submitted_id << endl;
		logmsg << "   Symbol:         " << "\t" << pParms->symbol << endl;
		logmsg << "   Trade Type ID:  " << "\t" << pParms->trade_type_id << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TTradeStatusTxnInput * pParms)
	{

		ostringstream logmsg;
		// Construct Log String
		logmsg << "TS Trade Status Txn:" << endl;
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}


	string CHPTxnLogFormatter::GetLogOutput(TTradeUpdateTxnInput * pParms)
	{

		int idx = 0;
 
		ostringstream logmsg;
		// Construct Log String
		logmsg << "TU Trade Update Txn:" << endl;
		for (idx = 0; idx < TradeLookupFrame1MaxRows; idx++)
		{
			if (pParms->trade_id[idx] != 0)
			{
				logmsg << "   Trade ID " << idx << ":    " << "\t" << pParms->trade_id[idx] << endl;
			}
		}
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << "   Max Acct ID:    " << "\t" << pParms->max_acct_id << endl;
		logmsg << "   Frame To Exec:  " << "\t" << pParms->frame_to_execute << endl;
		logmsg << "   Max Trades:     " << "\t" << pParms->max_trades << endl;
		logmsg << "   Max Updates:    " << "\t" << pParms->max_updates << endl;
		logmsg << "   End Trade Dt:   " << "\t" << FormatTS(pParms->end_trade_dts, datetime_fmtcode) << endl;
		logmsg << "   Start Trade Dt: " << "\t" << FormatTS(pParms->start_trade_dts, datetime_fmtcode) << endl;
		logmsg << "   Symbol:         " << "\t" << pParms->symbol << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TMarketFeedTxnInput * pParms)
	{

		int idx = 0;

		ostringstream logmsg;
		// Construct Log String
		logmsg << "MF Market Feed Txn:" << endl;
		logmsg << "   Unique Symbols: " << "\t" << pParms->unique_symbols << endl;
		logmsg << "   Status Submitd: " << "\t" << pParms->StatusAndTradeType.status_submitted << endl;
		logmsg << "   Type Limit Buy: " << "\t" << pParms->StatusAndTradeType.type_limit_buy << endl;
		logmsg << "   Type Limit Buy: " << "\t" << pParms->StatusAndTradeType.type_limit_buy << endl;
		logmsg << "   Type Stop Loss: " << "\t" << pParms->StatusAndTradeType.type_stop_loss << endl;
		for (idx = 0; idx < max_feed_len; idx++)
		{
			if (pParms->Entries[idx].trade_qty != 0)
			{
				logmsg << "   Price Quote " << idx << ": " << "\t" << setprecision(2) << pParms->Entries[idx].price_quote << endl;
				logmsg << "   Trade Qty " << idx << ": " << "\t" << pParms->Entries[idx].trade_qty << endl;
				logmsg << "   Symbol " << idx << ": " << "\t" << pParms->Entries[idx].symbol << endl;
			}
		}
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TDataMaintenanceTxnInput * pParms)
	{

		// Construct Log String
		ostringstream logmsg;
		logmsg << "DM Data Maintenance Txn:" << endl;
		logmsg << "   Acct ID:        " << "\t" << pParms->acct_id << endl;
		logmsg << "   C ID:           " << "\t" << pParms->c_id << endl;
		logmsg << "   Co ID:          " << "\t" << pParms->co_id << endl;
		logmsg << "   Day of Month:   " << "\t" << pParms->day_of_month << endl;
		logmsg << "   Vol Incr:       " << "\t" << pParms->vol_incr << endl;
		logmsg << "   Symbol:         " << "\t" << pParms->symbol << endl;
		logmsg << "   Table Name:     " << "\t" << pParms->table_name << endl;
		logmsg << "   Tx ID:          " << "\t" << pParms->tx_id << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

	string CHPTxnLogFormatter::GetLogOutput(TTradeCleanupTxnInput * pParms)
	{

		// Construct Log String
		ostringstream logmsg;
		logmsg << "TC Trade Cleanup Txn:" << endl;
		logmsg << "   Start Trade ID: " << "\t" << pParms->start_trade_id << endl;
		logmsg << "   St Canceled ID: " << "\t" << pParms->st_canceled_id << endl;
		logmsg << "   St Pending ID:  " << "\t" << pParms->st_pending_id << endl;
		logmsg << "   St Submitted ID:" << "\t" << pParms->st_submitted_id << endl;
		logmsg << endl;

		// Send String to Logger
		return (logmsg.str());
	}

};

}   // namespace HPZREF

#endif //HP_TXNLOGFORMATTER_H
